//  
//  Copyright (C) 2009 Chris Szikszoy
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Linq;
using System.Collections.Generic;

using Docky.Widgets;

namespace WeatherDocklet
{

	public class WeatherTile : AbstractTileObject
	{
		
		string locationCode;

		public WeatherTile (string location, string code)
		{
			Name = location;
			locationCode = code;
			if (location != code)
				Description = code;
			Enabled = WeatherPreferences.Locations.Contains (code);
		}
		
		public override void OnActiveChanged ()
		{
			List<string> currentLocations = WeatherPreferences.Locations.ToList ();
			if (Enabled) {
				currentLocations.Remove (locationCode);
			} else {
				currentLocations.Add (locationCode);
			}
			
			WeatherPreferences.Locations = currentLocations.ToArray ();
			
			Enabled = !Enabled;
			base.OnActiveChanged ();
		}
	}
}
