// ------------------------------------------------------------------------------
//  <autogenerated>
//      This code was generated by a tool.
//      
// 
//      Changes to this file may cause incorrect behavior and will be lost if 
//      the code is regenerated.
//  </autogenerated>
// ------------------------------------------------------------------------------

namespace WeatherDocklet {
    
    
    public partial class WeatherConfig {
        
        private Gtk.VBox vbox1;
        
        private Gtk.HBox hbox1;
        
        private Gtk.Label label1;
        
        private Gtk.ComboBox provider;
        
        private Docky.Widgets.WrapLabel provider_info;
        
        private Gtk.HBox hbox2;
        
        private Gtk.Button my_locations;
        
        private Docky.Widgets.SearchEntry location_code;
        
        private Gtk.Button search;
        
        private Gtk.ScrolledWindow scrolledwindow1;
        
        private Docky.Widgets.TileView results_view;
        
        private Gtk.HBox hbox3;
        
        private Gtk.Label label2;
        
        private Gtk.Label label3;
        
        private Gtk.SpinButton auto_update;
        
        private Gtk.HBox hbox4;
        
        private Gtk.CheckButton metric_units;
        
        protected virtual void Build() {
            Stetic.Gui.Initialize(this);
            // Widget WeatherDocklet.WeatherConfig
            Stetic.BinContainer.Attach(this);
            this.Name = "WeatherDocklet.WeatherConfig";
            // Container child WeatherDocklet.WeatherConfig.Gtk.Container+ContainerChild
            this.vbox1 = new Gtk.VBox();
            this.vbox1.Spacing = 6;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox1 = new Gtk.HBox();
            this.hbox1.Name = "hbox1";
            this.hbox1.Spacing = 6;
            // Container child hbox1.Gtk.Box+BoxChild
            this.label1 = new Gtk.Label();
            this.label1.Name = "label1";
            this.label1.LabelProp = Mono.Unix.Catalog.GetString("_Weather Provider:");
            this.label1.UseUnderline = true;
            this.hbox1.Add(this.label1);
            Gtk.Box.BoxChild w1 = ((Gtk.Box.BoxChild)(this.hbox1[this.label1]));
            w1.Position = 0;
            w1.Expand = false;
            w1.Fill = false;
            // Container child hbox1.Gtk.Box+BoxChild
            this.provider = Gtk.ComboBox.NewText();
            this.provider.Name = "provider";
            this.hbox1.Add(this.provider);
            Gtk.Box.BoxChild w2 = ((Gtk.Box.BoxChild)(this.hbox1[this.provider]));
            w2.PackType = ((Gtk.PackType)(1));
            w2.Position = 1;
            w2.Expand = false;
            w2.Fill = false;
            this.vbox1.Add(this.hbox1);
            Gtk.Box.BoxChild w3 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox1]));
            w3.Position = 0;
            w3.Expand = false;
            w3.Fill = false;
            // Container child vbox1.Gtk.Box+BoxChild
            this.provider_info = new Docky.Widgets.WrapLabel();
            this.provider_info.Events = ((Gdk.EventMask)(256));
            this.provider_info.Name = "provider_info";
            this.provider_info.Wrap = false;
            this.vbox1.Add(this.provider_info);
            Gtk.Box.BoxChild w4 = ((Gtk.Box.BoxChild)(this.vbox1[this.provider_info]));
            w4.Position = 1;
            w4.Expand = false;
            w4.Fill = false;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox2 = new Gtk.HBox();
            this.hbox2.Name = "hbox2";
            this.hbox2.Spacing = 6;
            // Container child hbox2.Gtk.Box+BoxChild
            this.my_locations = new Gtk.Button();
            this.my_locations.CanFocus = true;
            this.my_locations.Name = "my_locations";
            this.my_locations.UseUnderline = true;
            this.my_locations.Label = Mono.Unix.Catalog.GetString("My _Locations");
            this.hbox2.Add(this.my_locations);
            Gtk.Box.BoxChild w5 = ((Gtk.Box.BoxChild)(this.hbox2[this.my_locations]));
            w5.Position = 0;
            w5.Expand = false;
            w5.Fill = false;
            // Container child hbox2.Gtk.Box+BoxChild
            this.location_code = new Docky.Widgets.SearchEntry();
            this.location_code.Name = "location_code";
            this.location_code.EmptyMessage = "Location Code, Zip, or City";
            this.location_code.Ready = true;
            this.location_code.HasFocus = false;
            this.hbox2.Add(this.location_code);
            Gtk.Box.BoxChild w6 = ((Gtk.Box.BoxChild)(this.hbox2[this.location_code]));
            w6.Position = 1;
            // Container child hbox2.Gtk.Box+BoxChild
            this.search = new Gtk.Button();
            this.search.CanFocus = true;
            this.search.Name = "search";
            this.search.UseUnderline = true;
            // Container child search.Gtk.Container+ContainerChild
            Gtk.Alignment w7 = new Gtk.Alignment(0.5F, 0.5F, 0F, 0F);
            // Container child GtkAlignment.Gtk.Container+ContainerChild
            Gtk.HBox w8 = new Gtk.HBox();
            w8.Spacing = 2;
            // Container child GtkHBox.Gtk.Container+ContainerChild
            Gtk.Image w9 = new Gtk.Image();
            w9.Pixbuf = Stetic.IconLoader.LoadIcon(this, "gtk-find", Gtk.IconSize.Menu, 16);
            w8.Add(w9);
            // Container child GtkHBox.Gtk.Container+ContainerChild
            Gtk.Label w11 = new Gtk.Label();
            w11.LabelProp = Mono.Unix.Catalog.GetString("_Search");
            w11.UseUnderline = true;
            w8.Add(w11);
            w7.Add(w8);
            this.search.Add(w7);
            this.hbox2.Add(this.search);
            Gtk.Box.BoxChild w15 = ((Gtk.Box.BoxChild)(this.hbox2[this.search]));
            w15.Position = 2;
            w15.Expand = false;
            w15.Fill = false;
            this.vbox1.Add(this.hbox2);
            Gtk.Box.BoxChild w16 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox2]));
            w16.Position = 2;
            w16.Expand = false;
            w16.Fill = false;
            // Container child vbox1.Gtk.Box+BoxChild
            this.scrolledwindow1 = new Gtk.ScrolledWindow();
            this.scrolledwindow1.CanFocus = true;
            this.scrolledwindow1.Name = "scrolledwindow1";
            this.scrolledwindow1.HscrollbarPolicy = ((Gtk.PolicyType)(2));
            this.scrolledwindow1.ShadowType = ((Gtk.ShadowType)(1));
            // Container child scrolledwindow1.Gtk.Container+ContainerChild
            Gtk.Viewport w17 = new Gtk.Viewport();
            w17.ShadowType = ((Gtk.ShadowType)(0));
            // Container child GtkViewport.Gtk.Container+ContainerChild
            this.results_view = new Docky.Widgets.TileView();
            this.results_view.Name = "results_view";
            this.results_view.IconSize = 24;
            w17.Add(this.results_view);
            this.scrolledwindow1.Add(w17);
            this.vbox1.Add(this.scrolledwindow1);
            Gtk.Box.BoxChild w20 = ((Gtk.Box.BoxChild)(this.vbox1[this.scrolledwindow1]));
            w20.Position = 3;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox3 = new Gtk.HBox();
            this.hbox3.Name = "hbox3";
            this.hbox3.Spacing = 6;
            // Container child hbox3.Gtk.Box+BoxChild
            this.label2 = new Gtk.Label();
            this.label2.Name = "label2";
            this.label2.LabelProp = Mono.Unix.Catalog.GetString("Automatically Update _Every");
            this.label2.UseUnderline = true;
            this.hbox3.Add(this.label2);
            Gtk.Box.BoxChild w21 = ((Gtk.Box.BoxChild)(this.hbox3[this.label2]));
            w21.Position = 0;
            w21.Expand = false;
            w21.Fill = false;
            // Container child hbox3.Gtk.Box+BoxChild
            this.label3 = new Gtk.Label();
            this.label3.Name = "label3";
            this.label3.LabelProp = Mono.Unix.Catalog.GetString("minutes");
            this.hbox3.Add(this.label3);
            Gtk.Box.BoxChild w22 = ((Gtk.Box.BoxChild)(this.hbox3[this.label3]));
            w22.PackType = ((Gtk.PackType)(1));
            w22.Position = 1;
            w22.Expand = false;
            w22.Fill = false;
            // Container child hbox3.Gtk.Box+BoxChild
            this.auto_update = new Gtk.SpinButton(5, 60, 5);
            this.auto_update.CanFocus = true;
            this.auto_update.Name = "auto_update";
            this.auto_update.Adjustment.PageIncrement = 10;
            this.auto_update.ClimbRate = 1;
            this.auto_update.Numeric = true;
            this.auto_update.Value = 5;
            this.hbox3.Add(this.auto_update);
            Gtk.Box.BoxChild w23 = ((Gtk.Box.BoxChild)(this.hbox3[this.auto_update]));
            w23.PackType = ((Gtk.PackType)(1));
            w23.Position = 2;
            w23.Expand = false;
            w23.Fill = false;
            this.vbox1.Add(this.hbox3);
            Gtk.Box.BoxChild w24 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox3]));
            w24.Position = 4;
            w24.Expand = false;
            w24.Fill = false;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox4 = new Gtk.HBox();
            this.hbox4.Name = "hbox4";
            this.hbox4.Spacing = 6;
            // Container child hbox4.Gtk.Box+BoxChild
            this.metric_units = new Gtk.CheckButton();
            this.metric_units.CanFocus = true;
            this.metric_units.Name = "metric_units";
            this.metric_units.Label = Mono.Unix.Catalog.GetString("Use _Metric Units");
            this.metric_units.Active = true;
            this.metric_units.DrawIndicator = true;
            this.metric_units.UseUnderline = true;
            this.hbox4.Add(this.metric_units);
            Gtk.Box.BoxChild w25 = ((Gtk.Box.BoxChild)(this.hbox4[this.metric_units]));
            w25.Position = 0;
            this.vbox1.Add(this.hbox4);
            Gtk.Box.BoxChild w26 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox4]));
            w26.Position = 5;
            w26.Expand = false;
            w26.Fill = false;
            this.Add(this.vbox1);
            if ((this.Child != null)) {
                this.Child.ShowAll();
            }
            this.label1.MnemonicWidget = this.provider;
            this.Hide();
            this.provider.Changed += new System.EventHandler(this.OnProviderChanged);
            this.my_locations.Clicked += new System.EventHandler(this.OnMyLocationsClicked);
            this.search.Clicked += new System.EventHandler(this.OnSearchClicked);
            this.auto_update.ValueChanged += new System.EventHandler(this.OnUpdateValueChanged);
            this.metric_units.Toggled += new System.EventHandler(this.OnMetricToggled);
        }
    }
}
