// ------------------------------------------------------------------------------
//  <autogenerated>
//      This code was generated by a tool.
//      
// 
//      Changes to this file may cause incorrect behavior and will be lost if 
//      the code is regenerated.
//  </autogenerated>
// ------------------------------------------------------------------------------

namespace GMail {
    
    
    public partial class GMailLabelConfig {
        
        private Gtk.VBox vbox1;
        
        private Gtk.HBox hbox1;
        
        private Docky.Widgets.SearchEntry label_entry;
        
        private Gtk.Button add_label;
        
        private Gtk.ScrolledWindow scrolledwindow1;
        
        private Docky.Widgets.ListOnlyTileView label_list;
        
        private Gtk.HBox hbox2;
        
        private Gtk.Label label1;
        
        private Gtk.Label label2;
        
        private Gtk.SpinButton check_interval;
        
        protected virtual void Build() {
            Stetic.Gui.Initialize(this);
            // Widget GMail.GMailLabelConfig
            Stetic.BinContainer.Attach(this);
            this.Name = "GMail.GMailLabelConfig";
            // Container child GMail.GMailLabelConfig.Gtk.Container+ContainerChild
            this.vbox1 = new Gtk.VBox();
            this.vbox1.Spacing = 6;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox1 = new Gtk.HBox();
            this.hbox1.Name = "hbox1";
            this.hbox1.Spacing = 6;
            // Container child hbox1.Gtk.Box+BoxChild
            this.label_entry = new Docky.Widgets.SearchEntry();
            this.label_entry.Name = "label_entry";
            this.label_entry.EmptyMessage = "Label Name";
            this.label_entry.Query = "";
            this.label_entry.Ready = true;
            this.label_entry.HasFocus = false;
            this.hbox1.Add(this.label_entry);
            Gtk.Box.BoxChild w1 = ((Gtk.Box.BoxChild)(this.hbox1[this.label_entry]));
            w1.Position = 0;
            // Container child hbox1.Gtk.Box+BoxChild
            this.add_label = new Gtk.Button();
            this.add_label.CanFocus = true;
            this.add_label.Name = "add_label";
            this.add_label.UseUnderline = true;
            // Container child add_label.Gtk.Container+ContainerChild
            Gtk.Alignment w2 = new Gtk.Alignment(0.5F, 0.5F, 0F, 0F);
            // Container child GtkAlignment.Gtk.Container+ContainerChild
            Gtk.HBox w3 = new Gtk.HBox();
            w3.Spacing = 2;
            // Container child GtkHBox.Gtk.Container+ContainerChild
            Gtk.Image w4 = new Gtk.Image();
            w4.Pixbuf = Stetic.IconLoader.LoadIcon(this, "gtk-add", Gtk.IconSize.Menu, 16);
            w3.Add(w4);
            // Container child GtkHBox.Gtk.Container+ContainerChild
            Gtk.Label w6 = new Gtk.Label();
            w6.LabelProp = Mono.Unix.Catalog.GetString("_Add Label");
            w6.UseUnderline = true;
            w3.Add(w6);
            w2.Add(w3);
            this.add_label.Add(w2);
            this.hbox1.Add(this.add_label);
            Gtk.Box.BoxChild w10 = ((Gtk.Box.BoxChild)(this.hbox1[this.add_label]));
            w10.Position = 1;
            w10.Expand = false;
            w10.Fill = false;
            this.vbox1.Add(this.hbox1);
            Gtk.Box.BoxChild w11 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox1]));
            w11.Position = 0;
            w11.Expand = false;
            w11.Fill = false;
            // Container child vbox1.Gtk.Box+BoxChild
            this.scrolledwindow1 = new Gtk.ScrolledWindow();
            this.scrolledwindow1.CanFocus = true;
            this.scrolledwindow1.Name = "scrolledwindow1";
            this.scrolledwindow1.HscrollbarPolicy = ((Gtk.PolicyType)(2));
            this.scrolledwindow1.ShadowType = ((Gtk.ShadowType)(1));
            // Container child scrolledwindow1.Gtk.Container+ContainerChild
            Gtk.Viewport w12 = new Gtk.Viewport();
            w12.ShadowType = ((Gtk.ShadowType)(0));
            // Container child GtkViewport.Gtk.Container+ContainerChild
            this.label_list = new Docky.Widgets.ListOnlyTileView();
            this.label_list.Name = "label_list";
            w12.Add(this.label_list);
            this.scrolledwindow1.Add(w12);
            this.vbox1.Add(this.scrolledwindow1);
            Gtk.Box.BoxChild w15 = ((Gtk.Box.BoxChild)(this.vbox1[this.scrolledwindow1]));
            w15.Position = 1;
            // Container child vbox1.Gtk.Box+BoxChild
            this.hbox2 = new Gtk.HBox();
            this.hbox2.Name = "hbox2";
            this.hbox2.Spacing = 6;
            // Container child hbox2.Gtk.Box+BoxChild
            this.label1 = new Gtk.Label();
            this.label1.Name = "label1";
            this.label1.LabelProp = Mono.Unix.Catalog.GetString("Check e_very:");
            this.label1.UseUnderline = true;
            this.hbox2.Add(this.label1);
            Gtk.Box.BoxChild w16 = ((Gtk.Box.BoxChild)(this.hbox2[this.label1]));
            w16.Position = 0;
            w16.Expand = false;
            w16.Fill = false;
            // Container child hbox2.Gtk.Box+BoxChild
            this.label2 = new Gtk.Label();
            this.label2.Name = "label2";
            this.label2.LabelProp = Mono.Unix.Catalog.GetString("minutes");
            this.hbox2.Add(this.label2);
            Gtk.Box.BoxChild w17 = ((Gtk.Box.BoxChild)(this.hbox2[this.label2]));
            w17.PackType = ((Gtk.PackType)(1));
            w17.Position = 1;
            w17.Expand = false;
            w17.Fill = false;
            // Container child hbox2.Gtk.Box+BoxChild
            this.check_interval = new Gtk.SpinButton(2, 1440, 1);
            this.check_interval.CanFocus = true;
            this.check_interval.Name = "check_interval";
            this.check_interval.Adjustment.PageIncrement = 10;
            this.check_interval.ClimbRate = 1;
            this.check_interval.Numeric = true;
            this.check_interval.Value = 15;
            this.hbox2.Add(this.check_interval);
            Gtk.Box.BoxChild w18 = ((Gtk.Box.BoxChild)(this.hbox2[this.check_interval]));
            w18.PackType = ((Gtk.PackType)(1));
            w18.Position = 2;
            w18.Expand = false;
            w18.Fill = false;
            this.vbox1.Add(this.hbox2);
            Gtk.Box.BoxChild w19 = ((Gtk.Box.BoxChild)(this.vbox1[this.hbox2]));
            w19.Position = 2;
            w19.Expand = false;
            w19.Fill = false;
            this.Add(this.vbox1);
            if ((this.Child != null)) {
                this.Child.ShowAll();
            }
            this.label1.MnemonicWidget = this.check_interval;
            this.Hide();
            this.add_label.Clicked += new System.EventHandler(this.AddLabelClicked);
            this.check_interval.ValueChanged += new System.EventHandler(this.OnIntervalValueChanged);
        }
    }
}
